package com.example.mmkv_androidx_suport;

import android.os.Environment;

import com.tencent.mmkv.MMKV;

import androidx.annotation.NonNull;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** MmkvAndroidxSuportPlugin */
public class MmkvAndroidxSuportPlugin implements FlutterPlugin, MethodCallHandler {
  public static final String KEY = "key";
  public static final String VALUE = "value";

  private final MMKV mmkv;

  public MmkvAndroidxSuportPlugin() {
    MMKV.initialize(Environment.getExternalStorageDirectory().getAbsolutePath());
    mmkv = MMKV.defaultMMKV();
  }

  @Override
  public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
    MMKV.initialize(flutterPluginBinding.getApplicationContext());
    final MethodChannel channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "flutter_mmkv_cache");
    channel.setMethodCallHandler(new MmkvAndroidxSuportPlugin());
  }

  // This static function is optional and equivalent to onAttachedToEngine. It supports the old
  // pre-Flutter-1.12 Android projects. You are encouraged to continue supporting
  // plugin registration via this function while apps migrate to use the new Android APIs
  // post-flutter-1.12 via https://flutter.dev/go/android-project-migration.
  //
  // It is encouraged to share logic between onAttachedToEngine and registerWith to keep
  // them functionally equivalent. Only one of onAttachedToEngine or registerWith will be called
  // depending on the user's project. onAttachedToEngine or registerWith must both be defined
  // in the same class.
  public static void registerWith(Registrar registrar) {
    MMKV.initialize(registrar.context());
    final MethodChannel channel = new MethodChannel(registrar.messenger(), "flutter_mmkv_cache");
    channel.setMethodCallHandler(new MmkvAndroidxSuportPlugin());
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    String key = call.argument(KEY);
    try {

      switch (call.method) {
        case "setBool":
          boolean setBoolStatus = mmkv.encode(key, (boolean) call.argument(VALUE));
          result.success(setBoolStatus);
          break;
        case "getBool":
          boolean getBoolStatus = mmkv.decodeBool(key);
          result.success(getBoolStatus);
          break;
        case "setInt":
          boolean setIntStatus = mmkv.encode(key, (int) call.argument(VALUE));
          result.success(setIntStatus);
          break;
        case "getInt":
          int getIntStatus = mmkv.decodeInt(key);
          result.success(getIntStatus);
          break;
        case "setLong":
          boolean setLongStatus = mmkv.encode(key, (long) call.argument(VALUE));
          result.success(setLongStatus);
          break;
        case "getLong":
          long getLongStatus = mmkv.decodeLong(key);
          result.success(getLongStatus);
          break;
        case "setDouble":
          boolean setDoubleStatus = mmkv.encode(key, (double) call.argument(VALUE));
          result.success(setDoubleStatus);
          break;
        case "getDouble":
          double getDoubleStatus = mmkv.decodeDouble(key);
          result.success(getDoubleStatus);
          break;
        case "setString":
          boolean setStringStatus = mmkv.encode(key, (String) call.argument(VALUE));
          result.success(setStringStatus);
          break;
        case "getString":
          String getStringStatus = mmkv.decodeString(key, "");
          result.success(getStringStatus);
          break;
        case "removeByKey":
          mmkv.removeValueForKey(key);
          result.success(true);
          break;
        case "getValueSizeForKey":
          result.success(mmkv.getValueSize(key));
          break;
        case "allKeys":
          result.success(mmkv.allKeys());
          break;
        case "clear":
          mmkv.clearAll();
          result.success(true);
          break;
        default:
          result.notImplemented();
          break;
      }

    } catch (Exception e) {
      result.error("Exception encountered", call.method, e);
    }
  }

  @Override
  public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
  }
}
